@@ -5,7 +5,7 @@
 
 """
 
-from starfish.agent.agent_base import AgentBase                         # noqa: F401
+# from starfish.agent.agent_base import AgentBase                         # noqa: F401
 from starfish.agent.agent_manager import AgentManager                   # noqa: F401
 from starfish.agent.memory_agent import MemoryAgent                     # noqa: F401
 from starfish.agent.remote_agent import RemoteAgent                     # noqa: F401
