@@ -5,14 +5,23 @@ Surfer Agent class to provide basic functionality for Ocean Agents
 In starfish-java, this is named as `RemoteAgent`
 
 """
+
 import logging
 import time
+from typing import (
+    Any,
+    Generic,
+    List,
+    Union
+)
 from urllib.parse import urljoin
 
 
 from eth_utils import remove_0x_prefix
+from mongoquery import Query
 
-from starfish.agent import AgentBase
+from starfish.account import Account
+from starfish.agent.agent_base import AgentBase
 from starfish.agent.services import Services
 from starfish.asset import (
     DataAsset,
@@ -28,11 +37,17 @@ from starfish.exceptions import (
 from starfish.job import Job
 from starfish.listing import Listing
 from starfish.middleware.remote_agent_adapter import RemoteAgentAdapter
+from starfish.network import Network
+from starfish.types import (
+    Authentication,
+    ListingData,
+    TAsset,
+    TRemoteAgent
+)
 from starfish.utils.did import (
     decode_to_asset_id,
     did_generate_random,
-    did_parse,
-    is_did
+    did_parse
 )
 
 SUPPORTED_SERVICES = {
@@ -48,15 +63,11 @@ SUPPORTED_SERVICES = {
 logger = logging.getLogger(__name__)
 
 
-class RemoteAgent(AgentBase):
+class RemoteAgent(AgentBase, Generic[TRemoteAgent]):
     """
 
     Remote Agent class allows to register, list, purchase and consume assets.
 
-    :param ocean: Ocean object that is being used.
-    :type ocean: :class:`.Ocean`
-    :param did: Optional did of the Surfer agent
-
     :param ddo: Optional ddo of the remote agent, if not provided the agent
         will automatically get the DDO from the network based on the DID.
 
@@ -71,7 +82,7 @@ class RemoteAgent(AgentBase):
     """
     service_types = SUPPORTED_SERVICES
 
-    def __init__(self, network, ddo, authentication=None):
+    def __init__(self, ddo: DDO, authentication: Authentication = None) -> None:
         self._authentication = authentication
 
         if isinstance(ddo, dict):
@@ -82,38 +93,72 @@ class RemoteAgent(AgentBase):
             pass
         else:
             raise ValueError(f'Unknown ddo type {ddo}')
-        AgentBase.__init__(self, network, ddo)
+        AgentBase.__init__(self, ddo)
 
         self._adapter = RemoteAgentAdapter()
 
     @staticmethod
-    def load(network, asset_agent_did_url, authentication=None):
-        ddo_text = None
+    def load(
+        agent_did_or_url: str,
+        network: Network = None,
+        username: str = None,
+        password: str = None,
+        authentication: Authentication = None
+    ) -> TRemoteAgent:
+        """
 
-        if is_did(asset_agent_did_url):
-            ddo_text = RemoteAgent.resolve_network_ddo(network, asset_agent_did_url)
-        if ddo_text is None:
-            ddo_text = RemoteAgent.resolve_url_ddo(asset_agent_did_url, authentication)
+        Load a remote agent using a did/url. The Agent must have a DDO service available.
+        If a DID is passed you also need to provide a network object to resolve the did
+        For the RemoteAgent object to resolve the type of services available.
+
+        :param str agent_did_or_url DID or URL of the agent to load.
+        :param Authentiaciton authentication Optional Authenentication object to use to access the new agent.
+
+        :return: A RemoteAgent Object or None if the agent cannot be accessed or cannot be found at the url.
+
+        """
+
+        if not authentication:
+            if username or password:
+                authentication = {
+                    'username': username,
+                    'password': password
+                }
+        ddo_text = None
+        if network:
+            ddo_text = network.resolve_agent(agent_did_or_url, authentication=authentication)
+        else:
+            ddo_text = RemoteAgent.resolve_url(agent_did_or_url, authentication=authentication)
 
         if ddo_text:
-            return RemoteAgent(network, ddo_text, authentication)
+            return RemoteAgent(ddo_text, authentication)
 
         return None
 
     @staticmethod
-    def register(network, register_account, ddo, authentication=None):
-        network.register_did(register_account, ddo.did, ddo.as_text())
-        return RemoteAgent(network, ddo.as_text(), authentication)
+    def register(network: Network, register_account: Account, ddo: DDO, authentication: Authentication = None) -> TRemoteAgent:
+        """
+        Register the agent on the network.
 
-    def register_asset(self, asset):
+        :param network: Network to use to register on the block chain
+        :param register_account: Account object to use to register the agent ddo
+        :param ddo: DDO object to use to register on the network
+        :param authentication: Authentication data needed to access this agent
+
+        :return: RemoteAgent object that has been registered on the network
+        """
+        network.register_did(register_account, ddo.did, ddo.as_text())
+        return RemoteAgent(ddo.as_text(), authentication)
+
+    def register_asset(self, asset: TAsset, create_provenance: bool = False) -> TAsset:
         """
 
         Register an asset with Surfer
 
         :param asset: asset object to register
         :type asset: :class:`.DataAsset` object to register
-        :param account: This is not used for this agent, so for compatibility it is left in
-        :type account: :class:`.Account` object to use for registration.
+        :param bool create_provenance: If set to True, then set the provenance metadata for this asset
+
 
         :return: A :class:`.AssetBase` object that has been registered, if failure then return None.
         :type: :class:`.AssetBase` class
@@ -122,8 +167,8 @@ class RemoteAgent(AgentBase):
 
             asset = DataAsset.create('test data asset', 'Some test data')
             listing_data = { 'price': 3.457, 'description': 'my data is for sale' }
-            agent = SurferAgent(network, ddo)
-            asset = agent.register_asset(asset, account)
+            agent = SurferAgent(ddo)
+            asset = agent.register_asset(asset, True)
             print(f'Asset DID is {asset.did}')
 
         """
@@ -136,11 +181,15 @@ class RemoteAgent(AgentBase):
         register_data = self._adapter.register_asset(asset.metadata_text, url, authorization_token)
         if register_data:
             asset_id = register_data['asset_id']
+            if asset.asset_id != asset_id:
+                raise ValueError(
+                    f' calculated asset_id {asset.asset_id} is not the same as the agent generated asset_id {asset_id}'
+                )
             did = f'{self._ddo.did}/{asset_id}'
             asset.set_did(did)
         return asset
 
-    def create_listing(self, listing_data, asset_did):
+    def create_listing(self, listing_data: ListingData, asset_did: str) -> Listing:
         """
 
         Create a listing on the market place for this asset
@@ -153,7 +202,7 @@ class RemoteAgent(AgentBase):
         For example::
 
             asset = DataAsset.create('test data asset', 'Some test data')
-            agent = SurferAgent(network)
+            agent = SurferAgent()
             asset = agent.register_asset(asset, account)
 
             listing_data = { 'price': 3.457, 'description': 'my data is for sale' }
@@ -175,7 +224,7 @@ class RemoteAgent(AgentBase):
         listing = Listing(self, data['id'], asset_did, data)
         return listing
 
-    def update_listing(self, listing):
+    def update_listing(self, listing: Listing) -> bool:
         """
 
         Update the listing to the agent server.
@@ -189,7 +238,7 @@ class RemoteAgent(AgentBase):
 
         return self._adapter.update_listing(listing.listing_id, listing.data, url, authorization_token)
 
-    def validate_asset(self, asset):
+    def validate_asset(self, asset: TAsset) -> bool:
         """
 
         Validate an asset
@@ -199,7 +248,7 @@ class RemoteAgent(AgentBase):
         """
         pass
 
-    def upload_asset(self, asset):
+    def upload_asset(self, asset: TAsset) -> bool:
 
         if not isinstance(asset, DataAsset):
             raise TypeError('Only DataAsset is supported')
@@ -213,7 +262,7 @@ class RemoteAgent(AgentBase):
 
         return self._adapter.upload_asset_data(asset_id, asset.data, url, authorization_token)
 
-    def download_asset(self, asset_did_id):
+    def download_asset(self, asset_did_id: str) -> TAsset:
         """
         Download an asset
 
@@ -234,14 +283,13 @@ class RemoteAgent(AgentBase):
         data = self._adapter.download_asset(asset_id, url, authorization_token)
         store_asset = self.get_asset(asset_id)
         asset = DataAsset(
-            store_asset.metadata,
-            store_asset.did,
-            data=data,
-            metadata_text=store_asset.metadata_text
+            store_asset.metadata_text,
+            data=data
         )
+        asset.set_did(store_asset.did)
         return asset
 
-    def get_listing(self, listing_id):
+    def get_listing(self, listing_id: str) -> Listing:
         """
         Return an listing on the listings id.
 
@@ -265,7 +313,7 @@ class RemoteAgent(AgentBase):
                 listing = Listing(self, data['id'], asset, data)
         return listing
 
-    def get_asset(self, asset_did_id):
+    def get_asset(self, asset_did_id: str) -> TAsset:
         """
 
         This is for compatability for Surfer calls to get an asset directly from Surfer
@@ -285,7 +333,7 @@ class RemoteAgent(AgentBase):
             asset = self._create_asset_from_read(asset_id, read_metadata)
         return asset
 
-    def get_listings(self):
+    def get_listings(self) -> List[ListingData]:
         """
         Returns all listings
 
@@ -307,7 +355,7 @@ class RemoteAgent(AgentBase):
                     listings.append(listing)
         return listings
 
-    def get_job(self, job_id):
+    def get_job(self, job_id: str) -> Job:
         """
 
         Get a job from the invoke service ( koi )
@@ -328,7 +376,7 @@ class RemoteAgent(AgentBase):
             job = Job(job_id, status, outputs)
         return job
 
-    def job_wait_for_completion(self, job_id, timeout_seconds=60, sleep_seconds=1):
+    def job_wait_for_completion(self, job_id: str, timeout_seconds: int = 60, sleep_seconds: int = 1) -> Union[Job, bool]:
         """
 
         Wait for a job to complete, with optional timeout seconds.
@@ -350,7 +398,7 @@ class RemoteAgent(AgentBase):
                 time.sleep(sleep_seconds)
         return False
 
-    def search_listings(self, text, sort=None, offset=100, page=0):
+    def search_listings(self, text: str, sort: Any = None, offset: int = 100, page: int = 0) -> List[ListingData]:
         """
 
         Search the off chain storage for an asset with the givien 'text'
@@ -373,8 +421,8 @@ class RemoteAgent(AgentBase):
         # TODO: implement search listing in remote
         pass
 
-    def purchase_asset(self, listing, account, purchase_id=None, status=None,
-                       info=None, agreement=None):
+    def purchase_asset(self, listing: Any, account: Any, purchase_id: str = None, status: str = None,
+                       info: Any = None, agreement: Any = None) -> bool:
         """
 
         Purchase an asset using it's listing and an account.
@@ -406,7 +454,7 @@ class RemoteAgent(AgentBase):
         authorization_token = self.get_authorization_token()
         return self._adapter.purchase_asset(purchase, url, authorization_token)
 
-    def is_access_granted_for_asset(self, asset, account, purchase_id=None):
+    def is_access_granted_for_asset(self, asset: Any, account: Any, purchase_id: str = None) -> bool:
         """
 
         Check to see if the account and purchase_id have access to the assed data.
@@ -423,7 +471,7 @@ class RemoteAgent(AgentBase):
         """
         return False
 
-    def get_asset_purchase_ids(self, asset):
+    def get_asset_purchase_ids(self, asset: Any) -> Any:
         """
 
         Returns as list of purchase id's that have been used for this asset
@@ -437,7 +485,7 @@ class RemoteAgent(AgentBase):
         """
         return []
 
-    def purchase_wait_for_completion(self, asset, account, purchase_id, timeoutSeconds):
+    def purchase_wait_for_completion(self, asset: Any, account: Any, purchase_id: str, timeoutSeconds: int) -> bool:
         """
 
             Wait for completion of the purchase
@@ -449,7 +497,7 @@ class RemoteAgent(AgentBase):
         """
         pass
 
-    def consume_asset(self, listing, account, purchase_id):
+    def consume_asset(self, listing: Any, account: Any, purchase_id: str) -> bool:
         """
         Consume the asset and download the data. The actual payment to the asset
         provider will be made at this point.
@@ -467,7 +515,7 @@ class RemoteAgent(AgentBase):
         """
         return False
 
-    def invoke(self, asset, inputs=None, is_async=False):
+    def invoke(self, asset: TAsset, inputs: Any = None, is_async: bool = False) -> Any:
         """
 
         Call an operation asset with inputs to execute a remote call.
@@ -495,13 +543,16 @@ class RemoteAgent(AgentBase):
         response = self._adapter.invoke(remove_0x_prefix(asset.asset_id), inputs, url, authorization_token)
         return response
 
-    def get_authorization_token(self):
+    def get_authorization_token(self) -> str:
 
         if self._authentication and 'token' in self._authentication:
             if self._authentication['token']:
                 return self._authentication['token']
 
-        url = self.get_endpoint('auth', 'token')
+        try:
+            url = self.get_endpoint('auth', 'token')
+        except ValueError:
+            url = None
         token = None
         if url and self._authentication and 'username' in self._authentication:
             token = self._adapter.get_authorization_token(
@@ -514,13 +565,31 @@ class RemoteAgent(AgentBase):
 
         return token
 
-    def get_metadata_list(self):
+    def get_metadata_list(self) -> Any:
         url = self.get_endpoint('meta')
         authorization_token = self.get_authorization_token()
         return self._adapter.get_metadata_list(url, authorization_token)
 
+    def search_asset(self, filter_values: Any) -> List[str]:
+        """
+            :param dict filter_values: can be a dict or a mongo type query
+            :returns: a list of asset_ids found
+        """
+
+        if not isinstance(filter_values, dict):
+            raise TypeError('Filter values must be a type dict')
+
+        asset_list = self.get_metadata_list()
+        filter_query = Query(filter_values)
+        result = []
+        if asset_list:
+            for asset_id, metadata in asset_list.items():
+                if filter_query.match(metadata):
+                    result.append(asset_id)
+        return result
+
     @property
-    def ddo(self):
+    def ddo(self) -> DDO:
         """
 
         Return the registered DDO for this agent
@@ -531,7 +600,7 @@ class RemoteAgent(AgentBase):
         return self._ddo
 
     @staticmethod
-    def is_did_valid(did):
+    def is_did_valid(did: str) -> bool:
         """
         Checks to see if the DID string is a valid DID for this type of address for an asset.
         This method only checks the syntax of the DID, it does not resolve the DID
@@ -546,7 +615,12 @@ class RemoteAgent(AgentBase):
         return data['path'] and data['id_hex']
 
     @staticmethod
-    def generate_ddo(base_url_or_services, service_list=None, is_add_proof=False):
+    def generate_ddo(
+        base_url_or_services: Any,
+        service_list: Any = None,
+        all_services: bool = False,
+        is_add_proof: bool = False
+    ) -> DDO:
         """
         Generate a DDO for the remote agent url. This DDO will contain the supported
         endpoints for the remote agent
@@ -560,6 +634,9 @@ class RemoteAgent(AgentBase):
         :param dict service_list: Optional list of services to regsiter. This is only if the
             base_url_or_services is a string containing the base_url.
 
+        :param bool all_services: Optional False, If set to True register all available services.
+        :param bool is_add_proof: Optional False, If True add a proof signature to the ddo.
+
         :return: created DDO object assigned to the url of the remote agent service
         :type: :class:.`DDO`
 
@@ -578,7 +655,7 @@ class RemoteAgent(AgentBase):
         service_items = None
 
         if isinstance(base_url_or_services, str):
-            services = Services(base_url_or_services, service_list)
+            services = Services(base_url_or_services, service_list, all_services=all_services)
             service_items = services.as_dict
         elif isinstance(base_url_or_services, dict):
             service_items = base_url_or_services
@@ -600,7 +677,7 @@ class RemoteAgent(AgentBase):
 
         return ddo
 
-    def _create_asset_from_read(self, asset_id, read_metadata):
+    def _create_asset_from_read(self, asset_id: str, read_metadata: Any) -> Any:
         # check the hash of the reading asset
         asset_id = remove_0x_prefix(asset_id)
         if not is_asset_hash_valid(asset_id, read_metadata['hash']):
@@ -608,10 +685,11 @@ class RemoteAgent(AgentBase):
 
         did = f'{self._ddo.did}/{asset_id}'
         metadata_text = read_metadata['metadata_text']
-        asset = create_asset_from_metadata_text(metadata_text, did)
+        asset = create_asset_from_metadata_text(metadata_text)
+        asset.set_did(did)
         return asset
 
-    def get_endpoint(self, name, uri=None):
+    def get_endpoint(self, name: str, uri: str = None) -> str:
         """return the endpoint based on the name of the service or service type"""
         service_type = RemoteAgent.find_supported_service_type(name)
         if service_type is None:
@@ -636,32 +714,7 @@ class RemoteAgent(AgentBase):
         return endpoint
 
     @staticmethod
-    def find_supported_service_type(search_name_type):
-        """ return the supported service record if the name or service type is found
-        else return None """
-        for name, service_type in SUPPORTED_SERVICES.items():
-            if service_type == search_name_type:
-                return service_type
-            if name == search_name_type:
-                return service_type
-        return None
-
-    @staticmethod
-    def resolve_network_ddo(network, did):
-        """
-
-        Resolves the remote agent via the dnetwork using it's did to get the agent DDO
-
-        :param object network: DNetwork to resolve the did
-        :param str did: DID of the remote agent to resolve and get DDO
-        :return dict: DDO or None if not found in the network
-
-        """
-        if did:
-            return network.resolve_did(did)
-
-    @staticmethod
-    def resolve_url_ddo(url, authentication=None):
+    def resolve_url(url: str, authentication: Authentication = None) -> DDO:
         """
 
         Resolves the remote agent ddo using the url of the agent
@@ -686,3 +739,14 @@ class RemoteAgent(AgentBase):
 
             ddo = adapter.get_ddo(url, token)
         return ddo
+
+    @staticmethod
+    def find_supported_service_type(search_name_type: str) -> Any:
+        """ return the supported service record if the name or service type is found
+        else return None """
+        for name, service_type in SUPPORTED_SERVICES.items():
+            if service_type == search_name_type:
+                return service_type
+            if name == search_name_type:
+                return service_type
+        return None
