@@ -6,9 +6,9 @@
 """
 
 from starfish.contract.contract_manager import ContractManager                      # noqa: F401
+from starfish.contract.dex_token_contract import DexTokenContract                   # noqa: F401
 from starfish.contract.did_registry_contract import DIDRegistryContract             # noqa: F401
 from starfish.contract.direct_purchase_contract import DirectPurchaseContract       # noqa: F401
 from starfish.contract.dispenser_contract import DispenserContract                  # noqa: F401
 from starfish.contract.network_contract import NetworkContract                      # noqa: F401
-from starfish.contract.ocean_token_contract import OceanTokenContract               # noqa: F401
 from starfish.contract.provenance_contract import ProvenanceContract                # noqa: F401
