@@ -4,6 +4,8 @@
 
 """
 
+from starfish.account import Account
+
 from .contract_base import ContractBase
 
 CONTRACT_NAME = 'Dispenser'
@@ -12,9 +14,9 @@ CONTRACT_NAME = 'Dispenser'
 class DispenserContract(ContractBase):
     """Class representing the Ocean token dispenser contract."""
 
-    def __init__(self):
+    def __init__(self) -> None:
         ContractBase.__init__(self, CONTRACT_NAME)
 
-    def request_tokens(self, account, amount):
+    def request_tokens(self, account: Account, amount: float) -> bool:
         # amount is in ether
         return self.call('requestTokens', amount, account)
